// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

//go:build !runOnTask

package jobqueue

import (
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	"testing"
	"time"
)

func TestWriteToDB1(t *testing.T) {

	gormDB, err := gorm.Open(sqlite.Open("file::memory:?cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("a error occurred while opening the database: %v", err)
	}

	gormDB.Logger = gormDB.Logger.LogMode(4)

	manager := &Manager{database: gormDB}

	// Starte den DBSaver
	saver := NewJobSyncer(manager)

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job1", runner)

	scheduler := &TimeScheduler{
		Time: time.Now().Add(1 * time.Second),
		jobs: nil,
	}

	job.scheduler = scheduler

	saver.Sync(job)
	saver.Sync(job)

	manager.WaitSync()

	// check if stats are in database
	var stats JobPersistence
	err = gormDB.First(&stats, "id = ?", job.GetID()).Error
	assert.Nil(t, err)

	assert.Equal(t, job.GetID(), stats.ID)
	assert.Equal(t, job.GetID(), stats.ID)

}
