// Copyright 2024 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package jobqueue

import (
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	"testing"
)

func TestCreateOrUpdateJob(t *testing.T) {
	db, err := gorm.Open(sqlite.Open(":memory:"), &gorm.Config{})
	assert.NoError(t, err)

	// Migrate the schema
	assert.NoError(t, db.AutoMigrate(&JobPersistence{}, &JobLog{}, &JobStats{}))

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job1", runner)

	assert.NoError(t, saveJob(job, db))

	var jobPersistence JobPersistence
	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
	assert.Equal(t, job.GetID(), jobPersistence.ID)

	assert.Equal(t, "", jobPersistence.Description)
	assert.Equal(t, Priority(1), jobPersistence.Priority)
	job.description = "Updated description"
	assert.NoError(t, saveJob(job, db))
	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
	assert.Equal(t, "Updated description", jobPersistence.Description)

}
