{
  description = "Development environment";

  inputs = {
    nixpkgs = {url = "github:nixos/nixpkgs/nixos-23.11";}; 
    flake-utils = {url = "github:numtide/flake-utils";};
    gitlab-commit = {url = "git+https://gitlab.schukai.com/schukai/entwicklung/nix-flakes";};
  };

  outputs = {
    self,
    nixpkgs,
    flake-utils,
    gitlab-commit,
  }:
    flake-utils.lib.eachDefaultSystem (system: let
      inherit (nixpkgs.lib) optional;
      pkgs' = import nixpkgs {inherit system;};

      script-task = pkgs'.callPackage ./nix/scripts/go-task.nix {};
      script-run-tests = pkgs'.callPackage ./nix/scripts/run-tests.nix {};
      script-run-extended-tests = pkgs'.callPackage ./nix/scripts/run-extended-tests.nix {}; 
      script-run-ci-tests = pkgs'.callPackage ./nix/scripts/run-ci-tests.nix {};
      script-check-code = pkgs'.callPackage ./nix/scripts/check-code.nix {};
      script-build-cover-report = pkgs'.callPackage ./nix/scripts/build-cover-report.nix {};
      
      import-gitlab-commit = gitlab-commit.packages.${system}.git-commit;
      
    in {
      devShells = {
        default = let
          ciJobToken = builtins.getEnv "CI_JOB_TOKEN";
        in
          pkgs'.mkShell {
            nativeBuildInputs = with pkgs'; [
              script-run-tests
              script-run-extended-tests
              script-run-ci-tests
              script-check-code
              script-build-cover-report 
              script-task

              alejandra
              blackbox
              blackbox-terminal
              coreutils-full
              delve
              dialog
              drill
              fd
              feh
              gcc12
              gdlv
              git
              glab
              gnugrep
              gnumake
              gnused
              go
              go-licenses
              golangci-lint
              gosec
              gum
              jq
              niv
              libffi
              logrotate
              meld
              memcached
              netcat
              ranger
              unixtools.xxd
              unzip
              util-linux
              vhs
              wget
              zlib
            ]  ++ (
            if ciJobToken == "" 
            then [
               import-gitlab-commit
               ] 
            else []);
            
    
            shellHook = ''
              echo "Welcome to your development environment." 
            '';
          };
      };
    });
}
