{
  pkgs,
  lib,
  ...
}: let
  common = pkgs.callPackage ./common.nix {};
in
  pkgs.writeShellScriptBin "check-code" ''

    ${common}
    
    echo_header "Checking code"
    cd_working_dir

    echo_step "Running go vet"
    if ! ${pkgs.go}/bin/go vet .
    then
      echo_fail "go vet failed"
      exit 1
    fi

    echo_step "Running go fmt"
    if ! ${pkgs.go}/bin/go fmt .
    then
      echo_fail "go fmt failed"
      exit 1
    fi

    echo_step "Running go mod tidy"
    if ! ${pkgs.go}/bin/go mod tidy
    then
      echo_fail "go mod tidy failed"
      exit 1
    fi

    echo_step "Running lint"
    if ! ${pkgs.golangci-lint}/bin/golangci-lint run .
    then
      echo_fail "lint failed"
      exit 1
    fi

    echo_step "Running gosec"
    if ! ${pkgs.gosec}/bin/gosec .  
    then
      echo_fail "gosec failed"
      exit 1
    fi


  ''
